@@ -1,5 +1,5 @@
 /*
-    i801.c - Part of lm_sensors, Linux kernel modules for hardware
+    i2c-i801.c - Part of lm_sensors, Linux kernel modules for hardware
               monitoring
     Copyright (c) 1998 - 2002  Frodo Looijaard <frodol@dds.nl>,
     Philip Edelbrock <phil@netroedge.com>, and Mark D. Studebaker
@@ -31,6 +31,9 @@
     6300ESB		25A4   ("")
     ICH6		266A   ("")
     ICH7		27DA   ("")
+    ESB2 		269B   ("")
+    ICH8		283E   ("")
+
     This driver supports several versions of Intel's I/O Controller Hubs (ICH).
     For SMBus support, they are similar to the PIIX4 and are part
     of Intel's '810' and other chipsets.
@@ -57,15 +60,45 @@
 
 /* 82801CA is undefined before kernel 2.4.13 */
 #ifndef PCI_DEVICE_ID_INTEL_82801CA_3
-#define PCI_DEVICE_ID_INTEL_82801CA_3      0x2483
+#define PCI_DEVICE_ID_INTEL_82801CA_3	0x2483
 #endif
 
 /* 82801DB is undefined before kernel 2.4.19 */
 #ifndef PCI_DEVICE_ID_INTEL_82801DB_3
-#define PCI_DEVICE_ID_INTEL_82801DB_3      0x24c3
+#define PCI_DEVICE_ID_INTEL_82801DB_3	0x24c3
 #endif
 
-#ifdef I2C_FUNC_SMBUS_BLOCK_DATA_PEC
+/* 82801EB is undefined before kernel 2.4.21 */
+#ifndef PCI_DEVICE_ID_INTEL_82801EB_3
+#define PCI_DEVICE_ID_INTEL_82801EB_3	0x24d3
+#endif
+
+/* ESB is undefined before kernel 2.4.22 */
+#ifndef PCI_DEVICE_ID_INTEL_ESB_4
+#define PCI_DEVICE_ID_INTEL_ESB_4	0x25a4
+#endif
+
+/* ESB2 - Enterprise Southbridge is undefined */
+#ifndef PCI_DEVICE_ID_INTEL_ESB2_17
+#define PCI_DEVICE_ID_INTEL_ESB2_17	0x269b
+#endif
+
+/* ICH6 is undefined */
+#ifndef PCI_DEVICE_ID_INTEL_ICH6_16
+#define PCI_DEVICE_ID_INTEL_ICH6_16	0x266a
+#endif
+
+/* ICH7 is undefined */
+#ifndef PCI_DEVICE_ID_INTEL_ICH7_17
+#define PCI_DEVICE_ID_INTEL_ICH7_17	0x27da
+#endif
+
+/* ICH8 is undefined */
+#ifndef PCI_DEVICE_ID_INTEL_ICH8_5
+#define PCI_DEVICE_ID_INTEL_ICH8_5	0x283e
+#endif
+
+#ifdef I2C_CLIENT_PEC
 #define HAVE_PEC
 #endif
 
@@ -119,52 +152,49 @@ MODULE_PARM_DESC(force_addr,
 		 "EXTREMELY DANGEROUS!");
 
 static int i801_transaction(void);
-static int i801_block_transaction(union i2c_smbus_data *data,
-				  char read_write, int command);
+static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
+				  int command, int hwpec);
 
 static unsigned short i801_smba;
+static struct pci_driver i801_driver;
 static struct pci_dev *I801_dev;
 static int isich4;	/* is PEC supported? */
 static int isich5;	/* is i2c block read supported? */
 
-static int i801_setup(struct pci_dev *dev)
+static int __devinit i801_setup(struct pci_dev *dev)
 {
-	int error_return = 0;
 	unsigned char temp;
 
-	/* Note: we keep on searching until we have found 'function 3' */
-	if(PCI_FUNC(dev->devfn) != 3)
-		return -ENODEV;
-
 	I801_dev = dev;
 	if (dev->device == PCI_DEVICE_ID_INTEL_82801DB_3 ||
-	    dev->device == 0x24d3 ||
-	    dev->device == 0x25a4 ||
-	    dev->device == 0x266a ||
-	    dev->device == 0x27da)
+	    dev->device == PCI_DEVICE_ID_INTEL_82801EB_3 ||
+	    dev->device == PCI_DEVICE_ID_INTEL_ESB_4 ||
+	    dev->device == PCI_DEVICE_ID_INTEL_ESB2_17 ||	
+	    dev->device == PCI_DEVICE_ID_INTEL_ICH6_16 ||
+	    dev->device == PCI_DEVICE_ID_INTEL_ICH7_17 ||
+	    dev->device == PCI_DEVICE_ID_INTEL_ICH8_5)
 		isich4 = 1;
 	else
 		isich4 = 0;
 	isich5 = isich4 && dev->device != PCI_DEVICE_ID_INTEL_82801DB_3;
 
-	/* Determine the address of the SMBus areas */
+	/* Determine the address of the SMBus area */
 	if (force_addr) {
 		i801_smba = force_addr & 0xfff0;
 	} else {
 		pci_read_config_word(I801_dev, SMBBA, &i801_smba);
 		i801_smba &= 0xfff0;
 		if(i801_smba == 0) {
-			dev_err(dev, "SMB base address uninitialized"
+			dev_err(dev, "SMB base address uninitialized "
 				"- upgrade BIOS or use force_addr=0xaddr\n");
 			return -ENODEV;
 		}
 	}
 
-	if (!request_region(i801_smba, (isich4 ? 16 : 8), "i801-smbus")) {
+	if (!request_region(i801_smba, (isich4 ? 16 : 8), i801_driver.name)) {
 		dev_err(dev, "I801_smb region 0x%x already in use!\n",
 			i801_smba);
-		error_return = -EBUSY;
-		goto END;
+		return -EBUSY;
 	}
 
 	pci_read_config_byte(I801_dev, SMBHSTCFG, &temp);
@@ -174,17 +204,20 @@ static int i801_setup(struct pci_dev *dev)
 	/* If force_addr is set, we program the new address here. Just to make
 	   sure, we disable the device first. */
 	if (force_addr) {
-		pci_write_config_byte(I801_dev, SMBHSTCFG, temp & 0xfe);
+		pci_write_config_byte(I801_dev, SMBHSTCFG,
+				      temp & ~SMBHSTCFG_HST_EN);
 		pci_write_config_word(I801_dev, SMBBA, i801_smba);
-		pci_write_config_byte(I801_dev, SMBHSTCFG, temp | 0x01);
+		pci_write_config_byte(I801_dev, SMBHSTCFG,
+				      temp | SMBHSTCFG_HST_EN);
 		dev_warn(dev, "WARNING: I801 SMBus interface set to "
 			"new address %04x!\n", i801_smba);
-	} else if ((temp & 1) == 0) {
-		pci_write_config_byte(I801_dev, SMBHSTCFG, temp | 1);
+	} else if (!(temp & SMBHSTCFG_HST_EN)) {
+		pci_write_config_byte(I801_dev, SMBHSTCFG,
+				      temp | SMBHSTCFG_HST_EN);
 		dev_warn(dev, "enabling SMBus device\n");
 	}
 
-	if (temp & 0x02)
+	if (temp & SMBHSTCFG_SMB_SMI_EN)
 		dev_dbg(dev, "I801 using Interrupt SMI# for SMBus.\n");
 	else
 		dev_dbg(dev, "I801 using PCI Interrupt for SMBus.\n");
@@ -193,8 +226,7 @@ static int i801_setup(struct pci_dev *dev)
 	dev_dbg(dev, "SMBREV = 0x%X\n", temp);
 	dev_dbg(dev, "I801_smba = 0x%X\n", i801_smba);
 
-END:
-	return error_return;
+	return 0;
 }
 
 
@@ -204,7 +236,7 @@ static int i801_transaction(void)
 	int result = 0;
 	int timeout = 0;
 
-	dev_dbg(I801_dev, "Transaction (pre): CNT=%02x, CMD=%02x,"
+	dev_dbg(I801_dev, "Transaction (pre): CNT=%02x, CMD=%02x, "
 		"ADD=%02x, DAT0=%02x, DAT1=%02x\n", inb_p(SMBHSTCNT),
 		inb_p(SMBHSTCMD), inb_p(SMBHSTADD), inb_p(SMBHSTDAT0),
 		inb_p(SMBHSTDAT1));
@@ -212,7 +244,7 @@ static int i801_transaction(void)
 	/* Make sure the SMBus host is ready to start transmitting */
 	/* 0x1f = Failed, Bus_Err, Dev_Err, Intr, Host_Busy */
 	if ((temp = (0x1f & inb_p(SMBHSTSTS))) != 0x00) {
-		dev_dbg(I801_dev, "SMBus busy (%02x). Resetting... \n",
+		dev_dbg(I801_dev, "SMBus busy (%02x). Resetting...\n",
 			temp);
 		outb_p(temp, SMBHSTSTS);
 		if ((temp = (0x1f & inb_p(SMBHSTSTS))) != 0x00) {
@@ -258,7 +290,7 @@ static int i801_transaction(void)
 		outb_p(inb(SMBHSTSTS), SMBHSTSTS);
 
 	if ((temp = (0x1f & inb_p(SMBHSTSTS))) != 0x00) {
-		dev_dbg(I801_dev, "Failed reset at end of transaction"
+		dev_dbg(I801_dev, "Failed reset at end of transaction "
 			"(%02x)\n", temp);
 	}
 	dev_dbg(I801_dev, "Transaction (post): CNT=%02x, CMD=%02x, "
@@ -270,7 +302,7 @@ static int i801_transaction(void)
 
 /* All-inclusive block transaction function */
 static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
-				  int command)
+				  int command, int hwpec)
 {
 	int i, len;
 	int smbcmd;
@@ -336,7 +368,7 @@ static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
 		}
 		if (temp & errmask) {
 			dev_dbg(I801_dev, "SMBus busy (%02x). "
-				"Resetting... \n", temp);
+				"Resetting...\n", temp);
 			outb_p(temp, SMBHSTSTS);
 			if (((temp = inb_p(SMBHSTSTS)) & errmask) != 0x00) {
 				dev_err(I801_dev,
@@ -357,8 +389,8 @@ static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
 		/* We will always wait for a fraction of a second! */
 		timeout = 0;
 		do {
-			temp = inb_p(SMBHSTSTS);
 			i2c_delay(1);
+			temp = inb_p(SMBHSTSTS);
 		}
 		    while ((!(temp & 0x80))
 			   && (timeout++ < MAX_TIMEOUT));
@@ -417,13 +449,12 @@ static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
 			goto END;
 	}
 
-#ifdef HAVE_PEC
-	if(isich4 && command == I2C_SMBUS_BLOCK_DATA_PEC) {
+	if (hwpec) {
 		/* wait for INTR bit as advised by Intel */
 		timeout = 0;
 		do {
-			temp = inb_p(SMBHSTSTS);
 			i2c_delay(1);
+			temp = inb_p(SMBHSTSTS);
 		} while ((!(temp & 0x02))
 			   && (timeout++ < MAX_TIMEOUT));
 
@@ -432,7 +463,6 @@ static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
 		}
 		outb_p(temp, SMBHSTSTS); 
 	}
-#endif
 	result = 0;
 END:
 	if (command == I2C_SMBUS_I2C_BLOCK_DATA &&
@@ -453,8 +483,9 @@ static s32 i801_access(struct i2c_adapter * adap, u16 addr,
 	int ret, xact = 0;
 
 #ifdef HAVE_PEC
-	if(isich4)
-		hwpec = (flags & I2C_CLIENT_PEC) != 0;
+	hwpec = isich4 && (flags & I2C_CLIENT_PEC)
+		&& size != I2C_SMBUS_QUICK
+		&& size != I2C_SMBUS_I2C_BLOCK_DATA;
 #endif
 
 	switch (size) {
@@ -490,11 +521,6 @@ static s32 i801_access(struct i2c_adapter * adap, u16 addr,
 		break;
 	case I2C_SMBUS_BLOCK_DATA:
 	case I2C_SMBUS_I2C_BLOCK_DATA:
-#ifdef HAVE_PEC
-	case I2C_SMBUS_BLOCK_DATA_PEC:
-		if(hwpec && size == I2C_SMBUS_BLOCK_DATA)
-			size = I2C_SMBUS_BLOCK_DATA_PEC;
-#endif
 		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01),
 		       SMBHSTADD);
 		outb_p(command, SMBHSTCMD);
@@ -506,27 +532,19 @@ static s32 i801_access(struct i2c_adapter * adap, u16 addr,
 		return -1;
 	}
 
-#ifdef HAVE_PEC
-	if(isich4 && hwpec) {
-		if(size != I2C_SMBUS_QUICK &&
-		   size != I2C_SMBUS_I2C_BLOCK_DATA)
-			outb_p(1, SMBAUXCTL);	/* enable HW PEC */
-	}
-#endif
+	outb_p(hwpec, SMBAUXCTL);	/* enable/disable hardware PEC */
+
 	if(block)
-		ret = i801_block_transaction(data, read_write, size);
+		ret = i801_block_transaction(data, read_write, size, hwpec);
 	else {
 		outb_p(xact | ENABLE_INT9, SMBHSTCNT);
 		ret = i801_transaction();
 	}
 
-#ifdef HAVE_PEC
-	if(isich4 && hwpec) {
-		if(size != I2C_SMBUS_QUICK &&
-		   size != I2C_SMBUS_I2C_BLOCK_DATA)
-			outb_p(0, SMBAUXCTL);
-	}
-#endif
+	/* Some BIOSes don't like it when PEC is enabled at reboot or resume
+	   time, so we forcibly disable it after every transaction. */
+	if (hwpec)
+		outb_p(0, SMBAUXCTL);
 
 	if(block)
 		return ret;
@@ -567,12 +585,12 @@ static u32 i801_func(struct i2c_adapter *adapter)
 	    I2C_FUNC_SMBUS_BYTE_DATA | I2C_FUNC_SMBUS_WORD_DATA |
 	    I2C_FUNC_SMBUS_BLOCK_DATA | I2C_FUNC_SMBUS_WRITE_I2C_BLOCK
 #ifdef HAVE_PEC
-	     | (isich4 ? I2C_FUNC_SMBUS_BLOCK_DATA_PEC |
-	                 I2C_FUNC_SMBUS_HWPEC_CALC
-	               : 0)
+	     | (isich4 ? I2C_FUNC_SMBUS_HWPEC_CALC : 0)
 #endif
+#if 0
 	     | (isich5 ? I2C_FUNC_SMBUS_READ_I2C_BLOCK
 	               : 0)
+#endif
 	    ;
 }
 
@@ -586,7 +604,6 @@ static struct i2c_algorithm smbus_algorithm = {
 static struct i2c_adapter i801_adapter = {
 	.id		= I2C_ALGO_SMBUS | I2C_HW_SMBUS_I801,
 	.algo		= &smbus_algorithm,
-	.name		= "unset",
 	.inc_use	= i801_inc,
 	.dec_use	= i801_dec,
 };
@@ -624,25 +641,37 @@ static struct pci_device_id i801_ids[] __devinitdata = {
 	},
 	{
 		.vendor =	PCI_VENDOR_ID_INTEL,
-		.device =	0x24d3,	/* 82801EB ICH5 */
+		.device =	PCI_DEVICE_ID_INTEL_82801EB_3,
 		.subvendor =	PCI_ANY_ID,
 		.subdevice =	PCI_ANY_ID,
 	},
 	{
 		.vendor =	PCI_VENDOR_ID_INTEL,
-		.device =	0x25a4, /* PCI_DEVICE_ID_INTEL_ESB_4 */
+		.device =	PCI_DEVICE_ID_INTEL_ESB_4,
 		.subvendor =	PCI_ANY_ID,
 		.subdevice = 	PCI_ANY_ID,
 	},
 	{
 		.vendor =	PCI_VENDOR_ID_INTEL,
-		.device =	0x266a, /* PCI_DEVICE_ID_INTEL_ICH6_16 */
+		.device =	PCI_DEVICE_ID_INTEL_ESB2_17,
+		.subvendor =	PCI_ANY_ID,
+		.subdevice = 	PCI_ANY_ID,
+	},
+	{
+		.vendor =	PCI_VENDOR_ID_INTEL,
+		.device =	PCI_DEVICE_ID_INTEL_ICH6_16,
 		.subvendor =	PCI_ANY_ID,
 		.subdevice =	PCI_ANY_ID,
   	},
 	{
 		.vendor =	PCI_VENDOR_ID_INTEL,
-		.device =	0x27da,	/* ICH7 */
+		.device =	PCI_DEVICE_ID_INTEL_ICH7_17,
+		.subvendor =	PCI_ANY_ID,
+		.subdevice =	PCI_ANY_ID,
+	},
+	{
+		.vendor =	PCI_VENDOR_ID_INTEL,
+		.device =	PCI_DEVICE_ID_INTEL_ICH8_5,
 		.subvendor =	PCI_ANY_ID,
 		.subdevice =	PCI_ANY_ID,
 	},
@@ -651,12 +680,10 @@ static struct pci_device_id i801_ids[] __devinitdata = {
 
 static int __devinit i801_probe(struct pci_dev *dev, const struct pci_device_id *id)
 {
+	int err;
 
-	if (i801_setup(dev)) {
-		dev_warn(dev,
-			"I801 not detected, module not inserted.\n");
-		return -ENODEV;
-	}
+	if ((err = i801_setup(dev)))
+		return err;
 
 	snprintf(i801_adapter.name, 32,
 		"SMBus I801 adapter at %04x", i801_smba);
